@@ -33,7 +33,9 @@ use gstreamer_player as gst_player;
 #[cfg(feature = "mpris")]
 use crate::song::Song;
 #[cfg(feature = "mpris")]
-use crate::souvlaki::{MediaControlEvent, MediaControls, MediaMetadata, MediaPlayback, PlatformConfig};
+use crate::souvlaki::{
+    MediaControlEvent, MediaControls, MediaMetadata, MediaPlayback, PlatformConfig,
+};
 #[cfg(feature = "mpris")]
 use std::str::FromStr;
 #[cfg(feature = "mpris")]
@@ -56,7 +58,10 @@ impl GStreamer {
     pub fn new() -> Self {
         gst::init().expect("Couldn't initialize Gstreamer");
         let dispatcher = gst_player::PlayerGMainContextSignalDispatcher::new(None);
-        let player = gst_player::Player::new(None, Some(&dispatcher.upcast::<gst_player::PlayerSignalDispatcher>()));
+        let player = gst_player::Player::new(
+            None,
+            Some(&dispatcher.upcast::<gst_player::PlayerSignalDispatcher>()),
+        );
 
         #[cfg(feature = "mpris")]
         let config = PlatformConfig {
@@ -151,6 +156,14 @@ impl GStreamer {
         self.player.set_volume(f64::from(volume) / 100.0);
     }
 
+    // pub fn toggle_pause(&mut self) {
+    //     if self.paused {
+    //         self.resume();
+    //     } else {
+    //         self.pause();
+    //     }
+    // }
+
     pub fn pause(&mut self) {
         self.paused = true;
         self.player.pause();
