@@ -208,23 +208,23 @@ impl Song {
 
     // update_duration is only used for mp3 and wav, as other formats don't have length or
     // duration tag
-    #[allow(clippy::cast_possible_truncation)]
-    pub fn update_duration(&self) -> Result<()> {
-        let s = self.file().ok_or_else(|| anyhow!("no file found"))?;
+    // #[allow(clippy::cast_possible_truncation)]
+    // pub fn update_duration(&self) -> Result<()> {
+    //     let s = self.file().ok_or_else(|| anyhow!("no file found"))?;
 
-        if let Some("mp3" | "wav") = self.ext() {
-            let mut id3_tag = id3::Tag::new();
-            if let Ok(t) = id3::Tag::read_from_path(s) {
-                id3_tag = t;
-            }
+    //     if let Some("mp3" | "wav") = self.ext() {
+    //         let mut id3_tag = id3::Tag::new();
+    //         if let Ok(t) = id3::Tag::read_from_path(s) {
+    //             id3_tag = t;
+    //         }
 
-            let duration_player = GStreamer::duration(s);
-            id3_tag.remove_duration();
-            id3_tag.set_duration((duration_player.mseconds()) as u32);
-            let _drop = id3_tag.write_to_path(s, id3::Version::Id3v24);
-        }
-        Ok(())
-    }
+    //         let duration_player = GStreamer::duration(s);
+    //         id3_tag.remove_duration();
+    //         id3_tag.set_duration((duration_player.mseconds()) as u32);
+    //         let _drop = id3_tag.write_to_path(s, id3::Version::Id3v24);
+    //     }
+    //     Ok(())
+    // }
 
     pub fn save_tag(&mut self) -> Result<()> {
         match self.ext() {
@@ -283,7 +283,11 @@ impl Song {
 
         m4a_tag.set_artist(self.artist().unwrap_or(&String::from("Unknown Artist")));
         m4a_tag.set_title(self.title().unwrap_or(&String::from("Unknown Title")));
-        m4a_tag.set_album(self.album.as_ref().unwrap_or(&String::from("Unknown Album")));
+        m4a_tag.set_album(
+            self.album
+                .as_ref()
+                .unwrap_or(&String::from("Unknown Album")),
+        );
         m4a_tag.remove_lyrics();
 
         if !self.lyric_frames.is_empty() {
@@ -324,11 +328,20 @@ impl Song {
             }
         }
 
-        flac_tag.set_vorbis("Artist", vec![self.artist().unwrap_or(&String::from("Unknown Artist"))]);
-        flac_tag.set_vorbis("Title", vec![self.title().unwrap_or(&String::from("Unknown Title"))]);
+        flac_tag.set_vorbis(
+            "Artist",
+            vec![self.artist().unwrap_or(&String::from("Unknown Artist"))],
+        );
+        flac_tag.set_vorbis(
+            "Title",
+            vec![self.title().unwrap_or(&String::from("Unknown Title"))],
+        );
         flac_tag.set_vorbis(
             "Album",
-            vec![self.album.as_ref().unwrap_or(&String::from("Unknown Album"))],
+            vec![self
+                .album
+                .as_ref()
+                .unwrap_or(&String::from("Unknown Album"))],
         );
         flac_tag.remove_vorbis("Lyrics");
 
@@ -340,7 +353,11 @@ impl Song {
         }
 
         if let Some(p) = &self.picture {
-            flac_tag.add_picture(p.mime_type.clone(), metaflac::block::PictureType::Other, p.data.clone());
+            flac_tag.add_picture(
+                p.mime_type.clone(),
+                metaflac::block::PictureType::Other,
+                p.data.clone(),
+            );
         }
 
         let file = self.file().ok_or_else(|| anyhow!("no file found"))?;
@@ -465,14 +482,14 @@ impl Song {
                         ..lyric_frame.clone()
                     },
                 );
-            },
+            }
             None => {
                 lyric_frames.push(Lyrics {
                     lang: "eng".to_string(),
                     description: lang_ext.to_string(),
                     text: lyric_str.to_string(),
                 });
-            },
+            }
         }
         self.lyric_frames = lyric_frames;
     }
@@ -525,16 +542,27 @@ impl Song {
             picture = Some(p.clone());
         }
 
+        let mut duration = match id3_tag.duration() {
+            Some(d) => Duration::from_millis(d.into()),
+            None => Duration::from_secs(0),
+        };
+
+        let duration_player = GStreamer::duration(s);
         let mut id3_tag_duration = id3_tag.clone();
-        let duration = id3_tag.duration().map_or_else(
-            || {
-                let duration_player = GStreamer::duration(s);
+        let diff = duration.as_secs().checked_sub(duration_player.seconds());
+        if let Some(d) = diff {
+            if d > 1 {
+                id3_tag_duration.remove_duration();
                 id3_tag_duration.set_duration((duration_player.mseconds()) as u32);
                 let _drop = id3_tag_duration.write_to_path(s, id3::Version::Id3v24);
-                Duration::from_millis(duration_player.mseconds())
-            },
-            |d| Duration::from_millis(d.into()),
-        );
+                duration = Duration::from_millis(duration_player.mseconds());
+            }
+        } else {
+            id3_tag_duration.remove_duration();
+            id3_tag_duration.set_duration((duration_player.mseconds()) as u32);
+            let _drop = id3_tag_duration.write_to_path(s, id3::Version::Id3v24);
+            duration = Duration::from_millis(duration_player.mseconds());
+        }
 
         let file = Some(String::from(s));
 
@@ -597,16 +625,27 @@ impl Song {
             picture = Some(p.clone());
         }
 
+        let mut duration = match id3_tag.duration() {
+            Some(d) => Duration::from_millis(d.into()),
+            None => Duration::from_secs(0),
+        };
+
+        let duration_player = GStreamer::duration(s);
         let mut id3_tag_duration = id3_tag.clone();
-        let duration = id3_tag.duration().map_or_else(
-            || {
-                let duration_player = GStreamer::duration(s);
+        let diff = duration.as_secs().checked_sub(duration_player.seconds());
+        if let Some(d) = diff {
+            if d > 1 {
+                id3_tag_duration.remove_duration();
                 id3_tag_duration.set_duration((duration_player.mseconds()) as u32);
                 let _drop = id3_tag_duration.write_to_path(s, id3::Version::Id3v24);
-                Duration::from_millis(duration_player.mseconds())
-            },
-            |d| Duration::from_millis(d.into()),
-        );
+                duration = Duration::from_millis(duration_player.mseconds());
+            }
+        } else {
+            id3_tag_duration.remove_duration();
+            id3_tag_duration.set_duration((duration_player.mseconds()) as u32);
+            let _drop = id3_tag_duration.write_to_path(s, id3::Version::Id3v24);
+            duration = Duration::from_millis(duration_player.mseconds());
+        }
 
         let file = Some(String::from(s));
 
@@ -832,8 +871,8 @@ impl Song {
                         "LYRICS" | "lyrics" => lyrics_text = comment.1,
                         "METADATA_BLOCK_PICTURE" | "metadata_block_picture" => {
                             picture_encoded = comment.1;
-                        },
-                        _ => {},
+                        }
+                        _ => {}
                     }
                 }
             }
@@ -931,7 +970,7 @@ impl FromStr for Song {
                     parsed_lyric,
                     picture,
                 })
-            },
+            }
         }
     }
 }
