@@ -25,9 +25,6 @@
  * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
  * SOFTWARE.
  */
-#[cfg(feature = "mpris")]
-mod mpris;
-
 mod library;
 mod playlist;
 mod ueberzug;
@@ -54,25 +51,50 @@ use std::sync::RwLock;
 use std::thread::sleep;
 use std::time::Duration;
 use tui_realm_treeview::Tree;
-use tuirealm::{Payload, Value, View};
+use tuirealm::Application;
+use tuirealm::NoUserEvent;
+use tuirealm::View;
 use youtube_options::YoutubeOptions;
 
-// -- components
-const COMPONENT_LABEL_HELP: &str = "LABEL_HELP";
-const COMPONENT_PARAGRAPH_LYRIC: &str = "PARAGRAPH_LYRIC";
-const COMPONENT_TABLE_PLAYLIST: &str = "SCROLLTABLE_PLAYLIST";
-const COMPONENT_TABLE_YOUTUBE: &str = "SCROLLTABLE_YOUTUBE";
-const COMPONENT_TREEVIEW_LIBRARY: &str = "TREEVIEW";
-const COMPONENT_PROGRESS: &str = "PROGRESS";
-const COMPONENT_TEXT_HELP: &str = "TEXT_HELP";
-const COMPONENT_INPUT_URL: &str = "INPUT_URL";
-const COMPONENT_TEXT_ERROR: &str = "TEXT_ERROR";
-const COMPONENT_CONFIRMATION_RADIO: &str = "CONFIRMATION_RADIO";
-const COMPONENT_CONFIRMATION_INPUT: &str = "CONFIRMATION_INPUT";
-const COMPONENT_TEXT_MESSAGE: &str = "TEXT_MESSAGE";
-const COMPONENT_TABLE_SEARCH_LIBRARY: &str = "SEARCH_LIBRARY_TABLE";
-const COMPONENT_INPUT_SEARCH_LIBRARY: &str = "SEARCH_LIBRARY_INPUT";
+// const COMPONENT_LABEL_HELP: &str = "LABEL_HELP";
+// const COMPONENT_PARAGRAPH_LYRIC: &str = "PARAGRAPH_LYRIC";
+// const COMPONENT_TABLE_PLAYLIST: &str = "SCROLLTABLE_PLAYLIST";
+// const COMPONENT_TABLE_YOUTUBE: &str = "SCROLLTABLE_YOUTUBE";
+// const COMPONENT_TREEVIEW_LIBRARY: &str = "TREEVIEW";
+// const COMPONENT_PROGRESS: &str = "PROGRESS";
+// const COMPONENT_TEXT_HELP: &str = "TEXT_HELP";
+// const COMPONENT_INPUT_URL: &str = "INPUT_URL";
+// const COMPONENT_TEXT_ERROR: &str = "TEXT_ERROR";
+// const COMPONENT_CONFIRMATION_RADIO: &str = "CONFIRMATION_RADIO";
+// const COMPONENT_CONFIRMATION_INPUT: &str = "CONFIRMATION_INPUT";
+// const COMPONENT_TEXT_MESSAGE: &str = "TEXT_MESSAGE";
+// const COMPONENT_TABLE_SEARCH_LIBRARY: &str = "SEARCH_LIBRARY_TABLE";
+// const COMPONENT_INPUT_SEARCH_LIBRARY: &str = "SEARCH_LIBRARY_INPUT";
 
+// -- components
+#[derive(Debug, Eq, PartialEq, Clone, Hash)]
+pub enum Id {
+    HelpLabel,
+    LyricParagraph,
+    PlaylistTable,
+    YoutubeTable,
+    LibraryTreeview,
+    Progress,
+    HelpText,
+    UrlInput,
+    ErrorText,
+    ConfirmationRadio,
+    COnfirmationInput,
+    MessageText,
+    LibrarySearchTable,
+    LibrarySearchInput,
+}
+
+#[derive(Debug, PartialEq)]
+pub enum Msg {
+    AppClose,
+    None,
+}
 /// ### `ViewLayout`
 
 /// ## `MainActivity`
@@ -80,8 +102,10 @@ const COMPONENT_INPUT_SEARCH_LIBRARY: &str = "SEARCH_LIBRARY_INPUT";
 /// Main activity states holder
 pub struct TermusicActivity {
     exit_reason: Option<ExitReason>,
-    context: Option<Context>, // Context holder
-    view: View,               // View
+    // context: Option<Context>, // Context holder
+    // view: View,               // View
+    model: Model,
+    application: Application<Id, Msg, NoUserEvent>,
     redraw: bool,
     path: PathBuf,
     tree: Tree,
@@ -101,6 +125,10 @@ pub struct TermusicActivity {
     ueberzug: RwLock<Option<Child>>,
 }
 
+struct Model {
+    context: Context,
+}
+
 #[derive(Clone, Copy)]
 pub enum Status {
     Running,
@@ -133,10 +161,10 @@ pub enum StatusLine {
 impl Default for TermusicActivity {
     fn default() -> Self {
         // Initialize user input
-        let mut user_input_buffer: Vec<String> = Vec::with_capacity(16);
-        for _ in 0..16 {
-            user_input_buffer.push(String::new());
-        }
+        // let mut user_input_buffer: Vec<String> = Vec::with_capacity(16);
+        // for _ in 0..16 {
+        //     user_input_buffer.push(String::new());
+        // }
 
         let full_path = shellexpand::tilde(MUSIC_DIR);
         let p: &Path = Path::new(full_path.as_ref());
@@ -145,8 +173,12 @@ impl Default for TermusicActivity {
         let config = Termusic::default();
         Self {
             exit_reason: None,
-            context: None,
-            view: View::init(),
+            // context: None,
+            // view: View::init(),
+            model: Model {
+                context: Context::new(),
+            },
+            application: Application,
             redraw: true, // Draw at first `on_draw`
             tree: Tree::new(Self::dir_tree(p, 3)),
             path: p.to_path_buf(),
@@ -191,15 +223,17 @@ impl TermusicActivity {
                 }
                 self.status = Some(Status::Running);
                 self.next_song();
-            },
-            Some(Status::Running | Status::Paused) => {},
+            }
+            Some(Status::Running | Status::Paused) => {}
             None => self.status = Some(Status::Stopped),
         };
     }
 
     pub fn run_tageditor(&mut self) {
         let mut tageditor: TagEditorActivity = TagEditorActivity::default();
-        if let Some(Payload::One(Value::Str(node_id))) = self.view.get_state(COMPONENT_TREEVIEW_LIBRARY) {
+        if let Some(Payload::One(Value::Str(node_id))) =
+            self.view.get_state(COMPONENT_TREEVIEW_LIBRARY)
+        {
             let p: &Path = Path::new(node_id.as_str());
             if p.is_dir() {
                 self.mount_error("directory doesn't have tag!");
@@ -218,11 +252,11 @@ impl TermusicActivity {
                         error!("Failed to start TagEditorActivity: context is None");
                         return;
                     }
-                },
+                }
                 Err(e) => {
                     self.mount_error(format!("{}", e).as_ref());
                     return;
-                },
+                }
             };
         }
 
